#include "bodies.h"
#include "MyIntervalFunc.h"
#include "cuteCalc.h"

#include "cc-nb.h"  // to have global variables





#include "capd/capdlib.h"
#include "capd/mpcapdlib.h"
#include <capd/vectalg/Vector_inline.h>
#include <vector>
#include <future>
#include <thread>
#include <math.h>

using namespace std;
using namespace capd;






int Bodies::dimRedConfSpace() const
 // returns the dimension of the reduced configuration space
 // in the planar case we drop:  y_{n-2}, x_{n-1},y_{n-1}
 // in the spatial case we drop: z_0,y_{n-2},z_{n-2}, x_{n-1},y_{n-1},z_{n-1}
 {
 #ifdef PLANAR
   return 2*this->N -3;
 #endif
 #ifdef SPATIAL
    return 3*this->N -6;
 #endif
 }
   

Bodies::Bodies(int num_bodies)
 // creates structure with <num_bodies> bodies 
    {
        this->N = num_bodies;
        Body temp;
        for (int i = 0; i < this->N - 1; i++)
            this->data.push_back(temp);
    }
	

	
 #ifdef PLANAR
 Bodies::Bodies(int num_bodies,bool addOverSize, vector<MyDouble> &xInit, vector<MyDouble> &yInit,vector<MyDouble>  &massInit) 
 #endif
 #ifdef SPATIAL
 Bodies::Bodies(int num_bodies,bool addOverSize, vector<MyDouble> &xInit, vector<MyDouble> &yInit, vector<MyDouble> &zInit,vector<MyDouble>  &massInit)
 #endif 
	// creates from the data read from <xInit>,<yInit>,<zInit>, <massInit> which contain the input data regarding bodies 
	//  when addOverSize - then the initial data is a bit extended
	  
	// xInit,yInit,zInit,massInit - contain real numbers , x0.left,x0.right ,x1.left,x1.right, .... and analogously yInit,zInit,massInit
	//    we set x0=[x0.left,x0.right] etc
	  
    {
        N = num_bodies;

        MyInterval varTemp;
        MyInterval varTempX, varTempY;
      
        massN = 1.0;
        data.resize(N);
        Body b;
        
		
		MyInterval intervalOverSize=0;
		if(addOverSize){
		  #ifdef PLANAR
          intervalOverSize = MyInterval(-1e-4, 9.0 * 1e-4); // it is better when it is not symmetric
		  #endif
		  #ifdef SPATIAL
          // intervalOverSize = MyInterval(-1e-4, 9.0 * 1e-4);
          intervalOverSize = MyInterval(-5 * 1e-3, 2 * 1e-3); // it is better when it is not symmetric
		  #endif
		}
        massN = MyInterval(0.0);
        MyDouble temp1, temp2;

        for (int i = 0; i < N - 1; i++)
        {
            int id = 2 * i;
            temp1 = xInit.at(id);  // left end
            temp2 = xInit.at(id + 1);  // right end
            b.x = MyInterval(temp1, temp2) + intervalOverSize;
            temp1 = yInit.at(id);  // left end
            temp2 = yInit.at(id + 1);  // right end
            if (i == N - 2)
                b.y = MyInterval(0.0, 0.0);
            else
                b.y = MyInterval(temp1, temp2) + intervalOverSize;
				
			#ifdef SPATIAL
            temp1 = zInit.at(id); // left end
            temp2 = zInit.at(id + 1);  // right end
            if ((i == N - 2) || (i == 0))
                b.z = MyInterval(0.0, 0.0);
            else
            {
                b.z = MyInterval(temp1, temp2) + intervalOverSize;
            }
			#endif
            temp1 = massInit.at(id); // left end
            temp2 = massInit.at(id + 1);  // right end
            b.mass = MyInterval(temp1, temp2);
            massN += b.mass;
            data[i] = b;
        }
        massN = 1.0 - massN;
    }
 	
	
	
	
	
	
	MyInterval Bodies::xN() const{
    MyInterval sum = 0;
    for (int i = 0; i < N - 1; ++i)
    {
        Body b = data[i];
        sum += b.mass * b.x;
    }

    MyInterval temp = -sum / massN;
    MyInterval aprioriBound = MyInterval(-(N - 1), N - 1);
    myIntersection(temp, aprioriBound, temp);
    return temp;
	}

MyInterval Bodies::yN() const
{
    MyInterval sum = 0;
    for (int i = 0; i < N - 1; ++i)
    {
        Body b = data[i];
        sum += b.mass * b.y;
    }

    MyInterval temp = -sum / massN;
    MyInterval aprioriBound = MyInterval(-(N - 1), N - 1);
    myIntersection(temp, aprioriBound, temp);
    return temp;
}

#ifdef SPATIAL
MyInterval Bodies::zN() const
{
    MyInterval sum = 0;
    for (int i = 0; i < N - 1; ++i)
    {
        Body b = data[i];
        sum += b.mass * b.z;
    }

    MyInterval temp = -sum / massN;
    MyInterval aprioriBound = Interval(-(N - 1), N - 1);
    myIntersection(temp, aprioriBound, temp);
    return temp;
}
#endif


void Bodies::setX(int i,const MyInterval &val)
{
    if (i >= this->N - 1)
    {
        cout << "index_out_of_bounds" << endl;
        exit(1);
    }

    this->data[i].x = val;
}

#ifdef PLANAR
void Bodies::setY(int i, const MyInterval &val)
{

    if (i >= this->N - 1)
    {
        cout << "index_out_of_bounds" << endl;
        exit(1);
    }
    switch (normalizationMethod)
    {
    case NormalizationMethod::equalToFirst:
        this->data[i].y = val;
        if (i == 0)
            this->data[N - 2].y = this->data[0].y; // y_1 = y_{n-1}
        if (i == N - 2)
            this->data[0].y = this->data[N - 2].y;

        break;

    case NormalizationMethod::equalToZero:
        this->data[i].y = val;
        if (i == this->N - 2)
            this->data[N - 2].y = 0.0;

        break;

    default:
        break;
    }
}
#endif
#ifdef SPATIAL
void Bodies::setY(int i, const MyInterval &val)
{

    if (i >= N - 1)
    {
        cout << "index_out_of_bounds" << endl;
        exit(2);
    }
    switch (normalizationMethod)
    {
    case NormalizationMethod::equalToFirst:
        // TODO
        break;

    case NormalizationMethod::equalToZero:
        data[i].y = val;
        if (i == N - 2)
            data[N - 2].y = 0.0;

        break;

    default:
        break;
    }
}

void Bodies::setZ(int i,const MyInterval &val)
{

    if (i >= this->N - 1)
    {
        cout << "index_out_of_bounds" << endl;
        exit(2);
    }
    switch (normalizationMethod)
    {
    case NormalizationMethod::equalToFirst:
        // TODO
        break;

    case NormalizationMethod::equalToZero:
        data[i].z = val;
        if (i == 0 || i == N - 2)
            data[N - 2].z = 0.0;

        break;

    default:
        break;
    }
}
#endif

void Bodies::setMass(int i,const MyInterval &val)
{
    if (i >= this->N - 1)
    {
        cout << "index_out_of_bounds" << endl;
        exit(1);
    }

    this->data[i].mass = val;
}

    MyInterval Bodies::radius(int i, int j)  const
	#ifdef PLANAR
	{ return sqrt(power(this->x(i)-this->x(j), 2) + power(this->y(i)-this->y(j), 2)); }
    #endif
	#ifdef SPATIAL
	 { return sqrt(power(this->x(j) - this->x(i), 2) + power(this->y(j) - this->y(i), 2) + power(this->z(j) - this->z(i), 2));}
	#endif


  /** Derivatives */
  
	
	#ifdef PLANAR
	void Bodies::derivatives(int i, int j, MyInterval &dpx, MyInterval &dpy, MyInterval &ppxy) const
	#endif
	#ifdef SPATIAL
	void Bodies::derivatives(int i, int j, MyInterval &dpx, MyInterval &dpy, MyInterval &dpz,
                     MyInterval &ppxy, MyInterval &ppxz, MyInterval &ppyz) const
	#endif
	// it makes sense for i \neq j only
	// WARNING - the minus signs are important
//   dpx=-\frac{D^2 }{D x_i^2} 1/r_{ij}= \frac{D^2 }{D x_i Dx_j} 1/r_{ij}
//   dpy=-\frac{D^2 }{D y_i^2} 1/r_{ij}= \frac{D^2 }{Dy_i Dy_j} 1/r_{ij}	
//   dpz=-\frac{D^2 }{D z_i^2} 1/r_{ij}= \frac{D^2 }{D z_i D z_j} 1/r_{ij}
//   ppxy=\frac{D^2 }{Dx_i Dy_i} 1/r_{ij}=- \frac{D^2 }{D x_i y_j} 1/r_{ij}
//   ppxz=\frac{D^2 }{D x_i z_i} 1/r_{ij}=- \frac{D^2 }{D x_i z_j} 1/r_{ij}	
//   ppyz=\frac{D^2 }{D y_i z_i} 1/r_{ij}=- \frac{D^2 }{D y_i z_j} 1/r_{ij}				 
	// dpx = \frac{1}{r_{ij}^3} - 3\frac{(x_i - x_j)^2}{r_{ij}^5}
    // dpy = \frac{1}{r_{ij}^3} - 3\frac{(y_i - y_j)^2}{r_{ij}^5}
	// dpz = \frac{1}{r_{ij}^3} - 3\frac{(z_i - z_j)^2}{r_{ij}^5}
	// ppxy= 3 \frac{(x_i - x_j)(y_i - y_j)}{r_{ij}^5}
	// ppxz= 3 \frac{(x_i - x_j)(z_i - z_j)}{r_{ij}^5}
	// ppyz= 3 \frac{(y_i - y_j)(z_i - z_j)}{r_{ij}^5}
    {
        MyInterval qx, qy;
        MyInterval qx3, qx2, qy3, qy2;
		#ifdef SPATIAL
		MyInterval qz,qz2,qz3;
		#endif
		
        MyInterval r = radius(i, j);
        MyInterval r3 = power(r, -3);  //  r3=r^{-3}
        MyInterval dx = x(i) - x(j);
        MyInterval dy = y(i) - y(j);
		
		#ifdef PLANAR
		cuteCalculation( dx, dy,  qx3, qy3,  3, 1);   // qx3 = dx /r^{3}, qy3 = dy /r^{3}
        cuteCalculation( dx, dy,  qx2, qy2,  2, 1);   // qx2 = dx /r^{2}, qy2 = dy /r^{2}
        cuteCalculation( dx, dy,  qx, qy,  5, 2);  // qx = dx^2 /r^{5}, qy = dy^2 /r^{5}
		#endif
		
		#ifdef SPATIAL
        MyInterval dz = z(i) - z(j);
        cuteCalculation(dx, dy, dz, qx3, qy3, qz3, 3, 1);   // qx3 = dx /r^{3}, qy3 = dy /r^{3}, qz3 = dz/r^{3}
        cuteCalculation(dx, dy, dz, qx2, qy2, qz2, 2, 1);   // qx2 = dx /r^{2}, qy2 = dy /r^{2}, qz2 = dz/r^{3}
        cuteCalculation(dx, dy, dz, qx, qy, qz, 5, 2);  // qx = dx^2 /r^{5}, qy = dy^2 /r^{5}, qz = dz^2/r^{5}
		#endif
		
		
        dpx = r3 - 3.0 * qx;
        dpy = r3 - 3.0 * qy;
		ppxy = 3.0 * qx3 * qy2;
		
		#ifdef SPATIAL
        dpz = r3 - 3.0 * qz;
        ppxz = 3.0 * qx2 * qz3;
        ppyz = 3.0 * qy3 * qz2;
		#endif
    }
	
	
	bool Bodies::collisionHappened() const
	// true is returned when positions of some bodies overlap
{
    for (int i = 0; i < this->N; i++)
        for (int j = i + 1; j < this->N; j++)
        {
            if (isSingular(this->radius(i, j)))
                return true;
        }
    return false;
}


#ifdef PLANAR
bool Bodies::GXY(int i, MyInterval &sumX, MyInterval &sumY) const
#endif
#ifdef SPATIAL
bool Bodies::GXYZ(int i, MyInterval &sumX, MyInterval &sumY, MyInterval &sumZ) const
#endif
// computes xyz-coordinates of \sum_{j,j\neq i} \frac{m_j}{r_{ij}^3}(q_i - q_j)
// false is returned when the i-th body is in possible collision with other body, in such situation sumX,sumY,sumZ - contain some trash 
{

    sumX = 0.0;
    sumY = 0.0;
	#ifdef SPATIAL
	sumZ=0.0;
	#endif
	
	MyInterval qx, qy, qz,m;
	
    for (int j = 0; j <= N - 1; ++j)
    {
        if (j == i)
            continue;
        MyInterval r = radius(i, j);
        if (isSingular(r)) // collision between bodies i and j
            return false;
			
		m = mass(j);
	
		#ifdef PLANAR	
        cuteCalculation( x(i) - x(j), y(i) - y(j),  qx, qy,  3, 1); // qx=(x_i - x_j)/r_{ij}^3,  qy=(y_i - y_j)/r_{ij}^3
		#endif
		
		#ifdef SPATIAL
        cuteCalculation(x(i) - x(j), y(i) - y(j), z(i) - z(j), qx, qy, qz, 3, 1); // qx=(x_i - x_j)/r_{ij}^3,  qy=(y_i - y_j)/r_{ij}^3,  qz=(z_i - z_j)/r_{ij}^3
		#endif
		
        sumX += m * qx;
        sumY += m * qy;
		#ifdef SPATIAL
		sumZ += m * qz;
		#endif
    }
    return true;
}
#ifdef PLANAR
bool Bodies::fXY(int i, MyInterval &fXi, MyInterval &gXi, MyInterval &fYi, MyInterval &gYi) const
// computes xy-coordinates of  f_{q_i}= q_i- G_i ,
//  G_i=\sum_{j,j\neq i} \frac{m_j}{r_{ij}^3}(q_i - q_j)
// false is returned when the i-th body is in possible collision with other body, in such situation fXi, gXi, fYi, gYi, fZi, gZi - contain some trash
{

    if (GXY(i, gXi, gYi))
    {
        fXi = x(i) - gXi;
        fYi = y(i) - gYi;
        return true;
    }

    // collision between body i and some other body.
    return false;
}
#endif
#ifdef SPATIAL
bool Bodies::fXYZ(int i, MyInterval &fXi, MyInterval &gXi, MyInterval &fYi, MyInterval &gYi, MyInterval &fZi, MyInterval &gZi) const
// computes xyz-coordinates of  f_{q_i}= q_i- G_i ,
//  G_i=\sum_{j,j\neq i} \frac{m_j}{r_{ij}^3}(q_i - q_j)
// false is returned when the i-th body is in possible collision with other body, in such situation fXi, gXi, fYi, gYi, fZi, gZi - contain some trash
{

    if (GXYZ(i, gXi, gYi, gZi))
    {
        fXi = x(i) - gXi;
        fYi = y(i) - gYi;
        fZi = z(i) - gZi;
        return true;
    }

    // collision between body i-th and some other body.
    return false;
}

#endif

MyInterval Bodies::DxfX(int i, int k, MyMatrix &dpxM) const
/** Calculate: df^x_i/dx_k   -taking into account  the fact that q_{n-1} is computed from the center of mass condition */
// On input:
// dpxM[i][j]= \frac{1}{r_{ij}^3} - 3\frac{(x_i - x_j)^2}{r_{ij}^5}=-\frac{D^2 }{D x_i^2} 1/r_{ij}= \frac{D^2 }{D x_i Dx_j} 1/r_{ij}, i,j=0,..,N-1
// returns: 
// df^x_i/dx_k =   m_k \frac{D^2}{Dx_i Dx_k} 1/r_{ik} - m_k \frac{D^2}{Dx_i Dx_{N-1}} 1/r_{ik}  for $k\neq i$
// df^x_i/dx_i =  1 + \sum_{j \neq i, j<N-1} m_j \frac{D^2}{Dx_i^2} 1/r_{ik} +  m_{N-1}  (1+m_i/m_{N-1}) \frac{D^2}{Dx_i^2} 1/r_{i(N-1)}
{
    MyInterval sum(0.0, 0.0);

    if (i == k)
    {
        for (int j = 0; j < N - 1; j++)
        {
            if (j == i)
                continue;
            sum += mass(j) * dpxM[i][j];
        }
        sum += mass(N - 1) * (1 + mass(i) / mass(N - 1)) * dpxM[i][N - 1];
        return 1.0 - sum;
    }
    else // i != k
    {
        return mass(k) * (dpxM[i][k] - dpxM[i][N - 1]);
    }
}

MyInterval Bodies::DyfY(int i, int k, MyMatrix &dpyM) const
/** Calculate: df^y_i/dy_k  -taking into account  the fact that q_{n-1} is computed from the center of mass condition  */
// on input: dpyM[i][k] = 3\frac{(y_i - y_k)^2}{r_{ik}^5} - \frac{1}{r_{ik}^3=-\frac{D^2 }{D y_i^2} 1/r_{ij}= \frac{D^2 }{D y_i Dy_j} 1/r_{ij}, i,j=0,..,N-1
{
   return this->DxfX(i,k,dpyM);
}


MyInterval Bodies::DxfY(int i, int k, MyMatrix &ppM) const
/** Calculate: df_i^x/dy_k = df_i^y/dx_k  -taking into account  the fact that q_{n-1} is computed from the center of mass condition */
// On input: ppM[i][k] = 3\frac{(x_i - x_k)(y_i - y_k)}{r_{ik}^5}=\frac{D^2 }{Dx_i Dy_i} 1/r_{ij}=- \frac{D^2 }{D x_i y_j} 1/r_{ij}, i,k=0,1,..,N-1
//    df_i^x/dy_k= m_k \frac{D^2}{Dx_i D y_k}(1/r_{ik})- m_k \frac{D^2 }{Dx_i Dy_{n-1}}(1/r_{i(n-1)}), i \neq k
//    df_i^x/dy_i=\sum_{j\neq i, j<N-1} m_j \frac{D^2}{Dx_i D y_k}(1/r_{ik}) + m_{n-1}(1+m_i/m_{n-1}) \frac{D^2}{Dy_i Dx_i}(1/r_{i(N-1)}) ,i=k
{
    MyInterval sum(0.0, 0.0);

    if (i == k)
    {
        for (int j = 0; j < N - 1; j++)
        {
            if (j == i)
                continue;
            sum += mass(j) * ppM[i][j];
        }
        sum += mass(N - 1) * (1 + mass(i) / mass(N - 1)) * ppM[i][N - 1];
        return sum;
    }
    else
    { // i!= k
        return mass(k) * (ppM[i][N - 1] - ppM[i][k]);
    }
}



#ifdef SPATIAL
MyInterval Bodies::DzfZ(int i, int k, MyMatrix &dpzM) const
/** Calculate: df^z_i/dz_k  -taking into account  the fact that q_{n-1} is computed from the center of mass condition*/
// On input: dpzM[i][k] = 3\frac{(z_i - z_k)^2}{r_{ik}^5} - \frac{1}{r_{ik}^3=-\frac{D^2 }{D z_i^2} 1/r_{ij}= \frac{D^2 }{D z_i Dz_j} 1/r_{ij}, i,j=0,..,N-1
{
    return this->DxfX(i,k,dpzM);
}


MyInterval Bodies::DxfZ(int i, int k, MyMatrix &ppxzM) const
/** Calculate: df_i^x/dz_k = df_i^z/dx_k  -taking into account  the fact that q_{n-1} is computed from the center of mass condition*/
// On input: ppxzM[i][k] = 3\frac{(x_i - x_k)(z_i - z_k)}{r_{ik}^5}=\frac{D^2 }{Dx_i Dz_i} 1/r_{ij}=- \frac{D^2 }{D x_i z_j} 1/r_{ij}, i,k=0,1,..,N-1
{
   return this->DxfY(i,k,ppxzM);
}

MyInterval Bodies::DyfZ(int i, int k, MyMatrix &ppyzM) const
/** Calculate: df_i^y/dz_k = df_i^z/dy_k -taking into account  the fact that q_{n-1} is computed from the center of mass condition */
// On input: ppyzM[i][k] = 3\frac{(y_i - y_k)(z_i - z_k)}{r_{ik}^5}=\frac{D^2 }{Dy_i Dz_i} 1/r_{ij}=- \frac{D^2 }{D y_i z_j} 1/r_{ij}, i,k=0,1,..,N-1
{
    return this->DxfY(i,k,ppyzM);
}

#endif

MyDouble Bodies::maxDiam(int &max_i, int &coord) const
/**  we return the diameter (rigorous upper bound) of the largest  of active coordinates - 
	 we do not test the (N-1) body, which is computed from the center of mass condition
	 and the varaibles set to 0
   On output:
      min_i - is an index of the body with the smallest interval  
   coord = 0 =>  x, coord = 1 => y, coord = 2 => z **/
{
    max_i = 0;
    coord = 0; 
    MyDouble maxLengthInterval = diam(x(0));
    for (int i = 0; i <= N - 2; i++)
    {
        if (diam(x(i)) > maxLengthInterval)
        {
            max_i = i;
            coord = 0;
            maxLengthInterval = diam(x(i));
        };
        if (diam(y(i)) > maxLengthInterval)
        {
            max_i = i;
            coord = 1;
            maxLengthInterval = diam(y(i));
        }
		#ifdef SPATIAL
		  if (diam(z(i)) > maxLengthInterval)
        {
            max_i = i;
            coord = 2;
            maxLengthInterval = diam(z(i));
        }
		#endif
    }

    return maxLengthInterval;
}

MyDouble Bodies::minDiam(int &min_i, int &coord) const
/** we return the diameter of the smallest interval.
	 we do not test the (N-1) body, which is computed from the center of mass condition
	 and the varaibles set to 0
   On output:
      min_i - is an index of the body with the smallest interval  
   coord = 0 =>  x, coord = 1 => y, coord = 2 => z **/
{
    min_i = 0;
    coord = 0; 
    MyDouble minLengthInterval = diam(x(0));
    for (int i = 0; i <= N - 2; i++)
    {
        if (diam(x(i)) < minLengthInterval)
        {
            min_i = i;
            coord = 0;
            minLengthInterval = diam(x(i));
        };
		if(i!=N-2) // we skip y_{n-2}=0
		{
          if (diam(y(i)) < minLengthInterval)  
          {
            min_i = i;
            coord = 1;
            minLengthInterval = diam(y(i));
          }
		}
		
		#ifdef SPATIAL
		if((i != 0) && (i != N - 2)) // we skip z_0=0, z_{n-2}=0
		{
          if (diam(z(i)) < minLengthInterval)
          {
            min_i = i;
            coord = 2;
            minLengthInterval = diam(z(i));
          }
		}
		#endif
		
    }

    return minLengthInterval;
}




void Bodies::printData(bool writeDistFromZero /* =false*/) const
{
    for (int i = 0; i < N; ++i)
    {
        cout << "i: " << i << " ";
        cout << "X: " << x(i) << " ";
        cout << "Y: " << y(i) << " ";
		#ifdef SPATIAL
		cout << "Z: " << z(i) << " ";
		#endif
        cout << "mass: " << mass(i) << " ";
        cout << endl;
    }
    if (writeDistFromZero)
    {
        cout << "Distances from 0:" << endl;
        for (int i = 0; i < N; i++)
        {
            MyInterval dist0 = power(x(i), 2);
            dist0 += power(y(i), 2);
			#ifdef SPATIAL
			dist0 += power(z(i), 2);
			#endif
			
            cout << i << ":    " << sqrt(dist0) << endl;
        }
    }
}



void Bodies::printDiam() const
/** Debugging utility. */
{
    for (int i = 0; i < N; ++i)
    {
        cout << "i: " << i << " ";
        cout << "diam(X): " << diam(x(i)) << " ";
        cout << "diam(Y): " << diam(y(i)) << " ";
		#ifdef SPATIAL
		  cout << "diam(Z): " << diam(z(i)) << " ";
		#endif
        cout << endl
             << flush;
    }
    cout << endl;
}

bool Bodies::writeCCformatMath(ostream &mathFile, ostream &mathListFile) const
{
    MyDouble leftPoint, rightPoint, midPoint;
    mathListFile << "{";
    for (int i = 0; i < N; i++)
    {
	    leftPoint = x(i).leftBound();
        rightPoint = x(i).rightBound();
        midPoint = (leftPoint + rightPoint) / 2.0;
        mathFile << "x" << i << " = " << midPoint << "; ";
		mathListFile << "{";
        mathListFile << midPoint << ", ";


        leftPoint = y(i).leftBound();
        rightPoint = y(i).rightBound();
		midPoint = (leftPoint + rightPoint) / 2.0;
        mathFile << "y" << i << " = " << midPoint << "; ";
	   
		#ifdef SPATIAL
        mathListFile << midPoint << ", ";
        leftPoint = z(i).leftBound();
        rightPoint = z(i).rightBound();
        midPoint = (leftPoint + rightPoint) / 2.0;
        mathFile << "z" << i << " = " << midPoint << "; ";
		#endif
        mathListFile << midPoint << ((i < N - 1) ? ", " : "},");

        mathFile << endl;
        mathListFile << endl;
    }
    mathFile << endl;
    mathListFile << endl;

    return true;
}



void Bodies::writeBinaryData(ostream &outFile) const
{
   /* #ifdef SPATIAL
	 outFile.seekp(0, ios::end);  // nie wiem po co to bylo???
	#endif */
    outFile.write((char *)&N, sizeof(int));
    outFile.write((char *)&massN, sizeof(MyInterval));

    for (int i = 0; i < N - 1; i++)
    {
        outFile.write((char *)&(data[i]), sizeof(Body));
    }
    outFile.flush();
}

bool Bodies::readBinaryData(istream &inFile) 
/** Reads the data from a file.
  * Returns `false` on reading failure. */
{
    while (!data.empty())
        data.pop_back();

    inFile.read((char *)&N, sizeof(int));
    if (inFile.gcount() == 0)
        return false;

    inFile.read((char *)&massN, sizeof(MyInterval));

    Body temp;
    for (int i = 0; i < N - 1; i++)
    {
        inFile.read((char *)&temp, sizeof(Body));
        data.push_back(temp);
    }
    return true;
}


Bodies *Bodies::calculateMidBodies() const
//  the bodies locate in the center of <*this>
{
    MyDouble leftPoint, rightPoint, midPoint;
    Bodies *midBodies = new Bodies(*this);

    for (int i = 0; i < N - 1; i++)
    {
        leftPoint = x(i).leftBound();
        rightPoint = x(i).rightBound();
        midPoint = (leftPoint + rightPoint) / 2.0;
        midBodies->setX(i, midPoint);

        leftPoint = y(i).leftBound();
        rightPoint = y(i).rightBound();
        midPoint = (leftPoint + rightPoint) / 2.0;
        midBodies->setY(i, midPoint);
		
		#ifdef SPATIAL
		leftPoint = z(i).leftBound();
        rightPoint = z(i).rightBound();
        midPoint = (leftPoint + rightPoint) / 2.0;
        midBodies->setZ(i, midPoint);
		#endif
    }
		
    

    return midBodies;
}



#ifdef PLANAR
void fillMDf_auxFunc(const Bodies &bodies, MyMatrix &dpxMatrix, MyMatrix &dpyMatrix, MyMatrix &ppxyMatrix)
#endif
#ifdef SPATIAL
void fillMDf_auxFunc(const Bodies &bodies, MyMatrix &dpxMatrix, MyMatrix &dpyMatrix, MyMatrix &dpzMatrix,
                  MyMatrix &ppxyMatrix, MyMatrix &ppxzMatrix, MyMatrix &ppyzMatrix)
#endif
// computes some auxliary expresions needed for computation of Df; where f=0 is the equation we are solving
	// WARNING - the minus signs are important
//   dpxM[i,j]=-\frac{D^2 }{D x_i^2} 1/r_{ij}= \frac{D^2 }{D x_i Dx_j} 1/r_{ij}
//   dpyM[i,j]=-\frac{D^2 }{D y_i^2} 1/r_{ij}= \frac{D^2 }{Dy_i Dy_j} 1/r_{ij}	
//   dpzM[i,j]=-\frac{D^2 }{D z_i^2} 1/r_{ij}= \frac{D^2 }{D z_i D z_j} 1/r_{ij}
//   ppxyM[i,j]=\frac{D^2 }{Dx_i Dy_i} 1/r_{ij}=- \frac{D^2 }{D x_i y_j} 1/r_{ij}
//   ppxzM[i,j]=\frac{D^2 }{D x_i z_i} 1/r_{ij}=- \frac{D^2 }{D x_i z_j} 1/r_{ij}	
//   ppyzM[i,j]=\frac{D^2 }{D y_i z_i} 1/r_{ij}=- \frac{D^2 }{D y_i z_j} 1/r_{ij}	
// dpxM[i,j] = \frac{1}{r_{ij}^3} - 3\frac{(x_i - x_j)^2}{r_{ij}^5}
 // dpy[i,j] = \frac{1}{r_{ij}^3} - 3\frac{(y_i - y_j)^2}{r_{ij}^5}
	// dpz[i,j] = \frac{1}{r_{ij}^3} - 3\frac{(z_i - z_j)^2}{r_{ij}^5}
	// ppxy[i,j]= 3 \frac{(x_i - x_j)(y_i - y_j)}{r_{ij}^5}
	// ppxz[i,j]= 3 \frac{(x_i - x_j)(z_i - z_j)}{r_{ij}^5}
	// ppyz[i,j]= 3 \frac{(y_i - y_j)(z_i - z_j)}{r_{ij}^5}
	// diagonal entries are not used, and are set to 0
{
    int N = bodies.N;
	MyInterval dpx, dpy,ppxy;
	#ifdef SPATIAL
	MyInterval dpz, ppxz, ppyz;
	#endif

    for (int i = 0; i < N; i++)
    {
        for (int j = i; j < N; j++)
        {
            if (i == j)
            {
			     // this values are not needed
                dpxMatrix[i][i] = 0.0;
                dpyMatrix[i][i] = 0.0;
                ppxyMatrix[i][i] = 0.0;
				#ifdef SPATIAL
				dpzMatrix[i][i] = 0.0;
                ppxzMatrix[i][i] = 0.0;
                ppyzMatrix[i][i] = 0.0;
				#endif
            }
            else
            {
			    #ifdef PLANAR
			    bodies.derivatives(i, j, dpx, dpy,  ppxy);
                #endif
				#ifdef SPATIAL
				bodies.derivatives(i, j, dpx, dpy, dpz, ppxy, ppxz, ppyz);
				#endif
                dpxMatrix[i][j] = dpx;
                dpxMatrix[j][i] = dpx;

                dpyMatrix[i][j] = dpy;
                dpyMatrix[j][i] = dpy;

                ppxyMatrix[i][j] = ppxy;
                ppxyMatrix[j][i] = ppxy;
				
				#ifdef SPATIAL
				dpzMatrix[i][j] = dpz;
                dpzMatrix[j][i] = dpz;

                ppxzMatrix[i][j] = ppxz;
                ppxzMatrix[j][i] = ppxz;

                ppyzMatrix[i][j] = ppyz;
                ppyzMatrix[j][i] = ppyz;
				#endif
            }
        }
    }
}





#ifdef PLANAR
void vectorIndicesToNumberOfBodies(int i, int &iB, int &nB)
// From vector of reduced coordinates index <i> 
//  on output: we return <nB> \in (0,1) indicating respectively (x,y)
//                       <iB> the index of body
///////////// the identification is as follows
// i:        0 --  1 --  2 --  3 --  4 --  5 --  6 --  7 --  8
// bodies:  x0 -- y0 -- x1 -- y1 --  x2 -- y2 -- x3
//// IT WORKS on ly for the normalization EQUALTOZERO
{
  iB=i/2;
  nB=i % 2;
}

int fromBodiesToVectorIndices(int iB, int nB)
// this is an inverse function to <vectorIndicesToNumberOfBodies>
//  given the body <iB> and variable <nb> \in {0,1} (meaning (x,y))
// the function returns the corresponding index in the vector cointaining the reduced configuration space
{
  int ret=2*iB;
  if(nB==1) ret+=1;  // y -variable
  return ret;
}
#endif

#ifdef SPATIAL
void vectorIndicesToNumberOfBodies(int i, int &iB, int &nB)
// From vector of reduced coordinates index <i> 
//  on output: we return <nB> \in (0,1,2) indicating respectively (x,y,z)
//                       <iB> the index of body
///////////// the identification is as follows (i.e. z0 variable is removed)
// i:        0 --  1 --  2 --  3 --  4 --  5 --  6 --  7 --  8
// bodies:  x0 -- y0 -- x1 -- y1 -- z1 -- x2 -- y2 -- z2 -- x3
{

  if(i>1)  // this is one of the variables x1,y1,z1,x2,....
    i=i+1;  // to make up for the dropped coordinate z0
  
  iB=i/3;
  nB=i%3;	
}

int fromBodiesToVectorIndices(int iB, int nB)
// this is an inverse function to <vectorIndicesToNumberOfBodies>
//  given the body <iB> and variable <nb> \in {0,1,2} (meaning (x,y,z))
// the function returns the corresponding index in the vector cointaining the reduced configuration space
{
  int ret;
  ret=3*iB;
  if(nB==1) ret+=1;  // y -variable
  if(nB==2) ret+=2;  // z -variable
  if(iB>0) ret=ret-1; // because z0 variable is removed
  return ret;  
}

#endif

 MyVector Bodies::toReducedSpace() const
	// returns the vector giving <this> in the reduced space coordinates
{
  int dim=this->dimRedConfSpace();
  MyVector x0(dim);
  int iB,nB;
  for (int i = 0; i < dim; i++)
  {
    vectorIndicesToNumberOfBodies(i, iB, nB); // iB - the body corresponding to the index "i" and nB \in {0,1,2} meaning {x,y,z} respectively
    if(nB==0) // variable x_{iB}
      x0[i] = this->x(iB);
    if(nB==1) // variable y_{iB}
      x0[i] = this->y(iB);
    #ifdef SPATIAL
    if(nB==2) // variable z_{iB}
      x0[i] = this->z(iB);
    #endif 
   
  }	
  return x0;
}


 bool Bodies::compFRed(MyVector &FRed) const 
	// returns true if there is no colision and in such case fxi,fyi,fzi are put in FRed in a correct way
	// so that can be used in the Krawczyk or Newton operator
{
  
  int dimRCS=this->dimRedConfSpace();
  
  
  MyVector fxi(this->N),fyi(this->N);
  #ifdef SPATIAL
  MyVector  fzi(this->N);
  #endif
  
  
  MyInterval tempGXi, tempGYi, tempGZi; // dummy variables
  int i=0;
  bool noCollision=true;
  while(noCollision  &&  (i< (this->N - 1)) )  // we do not need fxyz for i=N-1
  {
    #ifdef SPATIAL
    noCollision=this->fXYZ(i, fxi[i], tempGXi,fyi[i] , tempGYi, fzi[i], tempGZi);  // variables tempGXi,tempGYi,tempGZi - will be not used 
	#endif
	#ifdef PLANAR
	noCollision=this->fXY(i, fxi[i], tempGXi,fyi[i] , tempGYi); // variables tempGXi,tempGYi,tempGZi - will be not used 
	#endif
	i++;
  }
  
  if(noCollision){

    int iB, nB;
  
    for (int i = 0; i < dimRCS; i++)
    {
      vectorIndicesToNumberOfBodies(i, iB, nB); // iB - the body corresponding to the index "i" and nB \in {0,1,2} meaning {x,y,z} respectively
	
      if(nB==0)// variable x_{iB}
		FRed[i] = fxi[iB];
      if(nB==1) // variable y_{iB}
        FRed[i] = fyi[iB];
      #ifdef SPATIAL
      if(nB==2) // variable z_{iB}
        FRed[i] = fzi[iB];
       #endif 
	}
  }

  return noCollision;
}

//  the equation for normalized central configuration can be written as
//  f_{x_i}=x_i + \sum_{j \neq i} m_j \frac{\partial }{\partial x_i} 1/r_{ij}=0
//  f_{y_i}=y_i + \sum_{j \neq i} m_j \frac{\partial }{\partial y_i} 1/r_{ij}=0
// in 3D :  f_{z_i}=z_i + \sum_{j \neq i} m_j \frac{\partial }{\partial z_i} 1/r_{ij}=0
void computeDFRedMatrix(const Bodies &bodies, MyMatrix &DF)
// On output:
//  DF - is the matrix of partial derivatives od fReduced with respect to all active variables
{

    int num_bodies=bodies.N;
	int dimRCS=bodies.dimRedConfSpace();
    #ifdef PLANAR
	 MyMatrix dpxM(num_bodies, num_bodies), dpyM(num_bodies, num_bodies), ppxyM(num_bodies, num_bodies);
    fillMDf_auxFunc(bodies, dpxM, dpyM, ppxyM);
	#endif

    #ifdef SPATIAL
    MyMatrix dpxM(num_bodies, num_bodies), dpyM(num_bodies, num_bodies), dpzM(num_bodies, num_bodies),
        ppxyM(num_bodies, num_bodies), ppxzM(num_bodies, num_bodies), ppyzM(num_bodies, num_bodies);

    fillMDf_auxFunc(bodies, dpxM, dpyM, dpzM, ppxyM, ppxzM, ppyzM);
	#endif
	// it makes sense for i \neq j only
	// WARNING - the minus signs are important
//   dpx=-\frac{D^2 }{D x_i^2} 1/r_{ij}= \frac{D^2 }{D x_i Dx_j} 1/r_{ij}
//   dpy=-\frac{D^2 }{D y_i^2} 1/r_{ij}= \frac{D^2 }{Dy_i Dy_j} 1/r_{ij}	
//   dpz=-\frac{D^2 }{D z_i^2} 1/r_{ij}= \frac{D^2 }{D z_i D z_j} 1/r_{ij}
//   ppxy=\frac{D^2 }{Dx_i Dy_i} 1/r_{ij}=- \frac{D^2 }{D x_i y_j} 1/r_{ij}
//   ppxz=\frac{D^2 }{D x_i z_i} 1/r_{ij}=- \frac{D^2 }{D x_i z_j} 1/r_{ij}	
//   ppyz=\frac{D^2 }{D y_i z_i} 1/r_{ij}=- \frac{D^2 }{D y_i z_j} 1/r_{ij}	
	// dpx = \frac{1}{r_{ij}^3} - 3\frac{(x_i - x_j)^2}{r_{ij}^5}
    // dpy = \frac{1}{r_{ij}^3} - 3\frac{(y_i - y_j)^2}{r_{ij}^5}
	// dpz = \frac{1}{r_{ij}^3} - 3\frac{(z_i - z_j)^2}{r_{ij}^5}
	// ppxy= 3 \frac{(x_i - x_j)(y_i - y_j)}{r_{ij}^5}
	// ppxz= 3 \frac{(x_i - x_j)(z_i - z_j)}{r_{ij}^5}
	// ppyz= 3 \frac{(y_i - y_j)(z_i - z_j)}{r_{ij}^5}	

    int bi, bj;
    int inB, jnB;
    for (int i = 0; i < dimRCS; ++i)
    {
        vectorIndicesToNumberOfBodies(i, bi, inB);  // bi - the index of the body, inB \in (0,1,2) meaning x,y,z respectively

        for (int j = 0; j < dimRCS; ++j)
        {
            vectorIndicesToNumberOfBodies(j, bj, jnB);  // bj - the index of the body, jnB \in (0,1,2) meaning x,y,z respectively

            //             case xx:
            if (inB == 0 && jnB == 0)
                DF[i][j] = bodies.DxfX(bi, bj, dpxM);

            //            case xy:
            if ((inB == 0 && jnB == 1) || (inB == 1 && jnB == 0))
                DF[i][j] = bodies.DxfY(bi, bj, ppxyM);

           
            //        case yy:
            if (inB == 1 && jnB == 1)
                DF[i][j] = bodies.DyfY(bi, bj, dpyM);
				
			#ifdef SPATIAL	
 //        case xz:
            if ((inB == 0 && jnB == 2) || (inB == 2 && jnB == 0))
                DF[i][j] = bodies.DxfZ(bi, bj, ppxzM);

            //        case yz:
            if ((inB == 1 && jnB == 2) || (inB == 2 && jnB == 1))
                DF[i][j] = bodies.DyfZ(bi, bj, ppyzM);

            //        case zz:
            if (inB == 2 && jnB == 2)
                DF[i][j] = bodies.DzfZ(bi, bj, dpzM);
			#endif	
        }
    }
}





MyDouble volume(const Bodies &bodies)
//  returns volume of <bodies> in the reduced configuration space
{
    if(normalizationMethod != NormalizationMethod::equalToZero){
	  cout << "Error: volume(Bodies) called with wrong normalizationMethod " << (int)normalizationMethod << "   should be=" << (int)NormalizationMethod::equalToZero << endl;
	  exit(1);
	}
	
    int dimRCS = bodies.dimRedConfSpace();
    MyVector box(dimRCS);
	box=bodies.toReducedSpace();
	
    MyDouble vol = 1.0;

    for (int i = 0; i < dimRCS; i++)
    {
        vol *= diam(box[i]);
    }

    return vol;
}


bool cubeSubSetInterior(const MyVector &Nx, const Bodies &bodies)
// returns true when in the reduced configuration spacd Nx \subset bodies
{
  int dimRCS=bodies.dimRedConfSpace();
  MyInterval X;
  int iB, nB;

  for (int i = 0; i < dimRCS; i++)
  {
    vectorIndicesToNumberOfBodies(i, iB, nB);
    switch (nB)
    {
    case 0: // variable x_{iB}
      X = bodies.x(iB);
      break;

    case 1: // variable y_{iB}
      X = bodies.y(iB);
      break;
    #ifdef SPATIAL
    case 2: // variable z_{iB}
      X = bodies.z(iB);
      break;
    #endif
    }

    if (!subSetInterior(Nx[i], X))
      return false;
  }

  return true;
}

bool cubeEmptyIntersection(const MyVector &Nx, const Bodies &bodies)
//  returns true iff Nx \cap reduced_coord(bodies) = \emptyset
{
  int dimRCS=bodies.dimRedConfSpace();
  MyInterval X;
  int iB, nB;

 

  for (int i = 0; i < dimRCS; i++)
  {
    vectorIndicesToNumberOfBodies(i, iB, nB);
    switch (nB)
    {
    case 0: // variable x_{iB}
      X = bodies.x(iB);
      break;

    case 1: // variable y_{iB}
      X = bodies.y(iB);
      break;
    #ifdef SPATIAL
    case 2: // variable z_{iB}
      X = bodies.z(iB);
      break;
	#endif  
    }

    if (emptyIntersection(Nx[i], X))
      return true; // empty intersection
  }

  return false; // non-empty intersection
}


bool cubeSuperSet(const MyVector Nx,const  Bodies &bodies)
// returns true iff in the reduced configuration space  bodies \subset Nx
{
  int dimRCS=bodies.dimRedConfSpace();
  MyInterval X;
  int iB, nB;
  
  for (int i = 0; i < dimRCS; i++)
  {
    vectorIndicesToNumberOfBodies(i, iB, nB);
    switch (nB)
    {
    case 0: // variable x_{iB}
      X = bodies.x(iB);
      break;
    case 1: // variable y_{iB}
      X = bodies.y(iB);
      break;
    #ifdef SPATIAL
    case 2: // variable z_{iB}
      X = bodies.z(iB);
      break;
	#endif  
    }

    if (!subSet(X,Nx[i]) )
      return false;
  }

  return true;
}


MyVector cubeIntersection(const MyVector Nx,const Bodies &bodies)
//  returns vector in the reduced space equal to Nx \cap Reduced(bodies)
// it should be checked before that this intersection is non-empty
{
  int dimRCS = bodies.dimRedConfSpace();
  MyVector X(dimRCS);
  MyVector intXY(dimRCS);
  
  X=bodies.toReducedSpace();


  for (int i = 0; i < dimRCS ; i++)
  { 
    myIntersection(Nx[i], X[i], intXY[i]);
  }

  return intXY;
}


void updateBodies(const MyVector &Nx, Bodies &bodies)
//  function does bodies=Nx where Nx is in the reduced configuration space
{
  int dimRCS=bodies.dimRedConfSpace();
  int iB, nB;
  
  for (int i = 0; i < dimRCS; i++)
  {
    vectorIndicesToNumberOfBodies(i, iB, nB);
    switch (nB)
    {
    case 0: // variable x_{iB}
      bodies.setX(iB,Nx[i]);
      break;
    case 1: // variable y_{iB}
       bodies.setY(iB,Nx[i]);
      break;
    #ifdef SPATIAL
    case 2: // variable z_{iB}
       bodies.setZ(iB,Nx[i]);
      break;
	#endif  
    }
  }
}


MyInterval potential(const Bodies &bodies)
// returns U = \sum_{i < j} \frac{m_i*m_j}{r_{i, j}} 
//   it is assumed that no collision happens (this should be check before calling this function)
{
    MyInterval rTemp;

    MyInterval sum = 0.0;
    for (int i = 0; i < bodies.N; i++)
    {
        for (int j = i + 1; j < bodies.N; j++)
        {
            rTemp = bodies.radius(i, j);
            if (isSingular(rTemp))
            {
                cout << "Collision!!!" << endl;
                exit(1);
            }

            sum += (bodies.mass(i) * bodies.mass(j)) / rTemp;
        }
    }

    return sum;
}

MyInterval momentInertia(const Bodies &bodies)
// returns I = \sum_{i} m_i*|q_i|^2 
{
    MyInterval rTemp;

    MyInterval sum = 0.0;
    const int N = bodies.N;
    for (int i = 0; i < N; i++)
    {
	    #ifdef PLANAR
        sum += bodies.mass(i) * (power(bodies.x(i), 2) + power(bodies.y(i), 2));
		#endif
		#ifdef SPATIAL
		sum += bodies.mass(i) * (power(bodies.x(i), 2) + power(bodies.y(i), 2) + power(bodies.z(i), 2));
		#endif
    }

    return sum;
}


void writePotAndInertia(const Bodies &bodies,bool writeMoeckelPot)
//  it makes to have writeMoeckelPot=true when masses are equal in 2D
//   otherwise we suggest false
{

    MyInterval U = potential(bodies);
    MyInterval I = momentInertia(bodies);
    MyInterval M = 0; // the sum of masses

    int N = bodies.N;
    for (int i = 0; i < N; i++)
        M += bodies.mass(i);

    MyInterval scalingInvariant = U * sqrt(I) / power(M, MyInterval(5.0) / 2.0);
    cout << endl;
    cout << "U = " << U << ", I = " << I
         << ", U*(I)^(1/2)/(M)^(5/2) = " << scalingInvariant
         << endl
         << flush;

    if (writeMoeckelPot)
    {
        MyInterval UM = scalingInvariant * power(MyInterval(N), MyInterval(5.0) / 2.0);
        cout << "Moeckel's potential = " << UM << endl
             << endl;
    }
}


void intervalHullBodies(const Bodies &sol1,const Bodies &sol2, Bodies &unionBodies)
// On output: unionBodies=IntervalHull(sol1,sol2)
{
    int N = sol1.N;

    for (int i = 0; i < N - 1; i++)
    {
        unionBodies.setX(i, intervalHull(sol1.x(i), sol2.x(i)));
        unionBodies.setY(i, intervalHull(sol1.y(i), sol2.y(i)));
		#ifdef SPATIAL
		unionBodies.setZ(i, intervalHull(sol1.z(i), sol2.z(i)));
		#endif
    }
}

void calculatePuffedBodies(const Bodies &bodies, MyInterval addedInterval, Bodies &puffedBodies)
/** "Puffs" the box of each body - by adding addedInterval, to each active variable in the reduced configuration space 
it assumed that <puffedBodies> has the same masses as <bodies>, so we do not copy the masses */
{
    for (int i = 0; i < bodies.N - 1; i++) // because the last body is calculated automatically from the center of mass condition
    {
        puffedBodies.setX(i, bodies.x(i) + addedInterval);
		if(i!=bodies.N-2)  // because y_{n-2}=0
          puffedBodies.setY(i, bodies.y(i) + addedInterval);
		#ifdef SPATIAL
		if((i!=0) && (i!=bodies.N-2))  // because z_0=z_{n-2}=0
		  puffedBodies.setZ(i, bodies.z(i) + addedInterval);
		#endif
    }    
}


int readBodiesFromBinaryFile(int numBodies, const char *nazwaPliku, vector<Bodies> &ccSolutions)
//  reads bodies from the binary file <nazwaPliku> and places them in ccSolutions
//   returns the number of bodies read
{
    int numberOfCC = 0;

    Bodies testBodies(numBodies);

    ifstream infile;
    infile.open(nazwaPliku, ios::binary | ios::in);
    if (infile.is_open())
    {
        while (testBodies.readBinaryData(infile))
        {
            ccSolutions.push_back(testBodies);
            numberOfCC++;
        }
        infile.close();
    }
    else
        cout << "File opening error!" << nazwaPliku << endl;
    return numberOfCC;
}


void permuteMassesOfBodies(Bodies &bodies, int perm[])
 // on output:
    // mass(bodies(i)) = mass(Bodies(input)(perm[i])) 
 // we do not change anything on coordinates, these must be set by some other function	

{
   
	
    int N = bodies.N;
    Bodies inBodies(bodies);
    for (int i = 0; i < N - 1; i++)
    {
        if (perm[i] != i)
        {
            bodies.setMass(i, inBodies.mass(perm[i]));
        }
    }

    MyInterval sum = MyInterval(0.0);
    for (int i = 0; i < N - 1; ++i)
    {
        sum += bodies.mass(i);
    }
    bodies.massN = 1.0 - sum;
}