
MyInterval radiusXYn(MyInterval x, MyInterval y, int exp)
{
    return power(sqrt(power(x, 2) + power(y, 2)), exp);
}

void setMinMax2D(MyInterval x, MyInterval y,
                 MyInterval &qx, MyInterval &qy)
{
    qx = intervalHull(x, qx);
    qy = intervalHull(y, qy);
}

void tempValues2D(MyInterval x, MyInterval y,
                  MyInterval &tempx, MyInterval &tempy,
                  int rExp, int bExp)
// tempx = x^bExp/r^rExp
// tempy = y^bExp/r^rExp
{
    MyInterval r = radiusXYn(x, y, -rExp);
    tempx = r * power(x, bExp);
    tempy = r * power(y, bExp);
}

bool intersectEdgeWithLine(MyInterval coef, MyInterval qx, MyInterval qy, int edge,
                           MyInterval &pointx, MyInterval &pointy)
// direction = 1 --- line: y = x*coef
// direction = -1 --- line: y = -x*ceof
// edge: 0 -- left edge, 1 -- right, 2 -- lower, 3 -- upper
{
    MyInterval temp;
    bool intersect;

    switch (edge)
    {
    case 0: // intersection with the left edge
        pointx = qx.leftBound();
        temp = pointx * coef;
        intersect = myIntersection(temp, qy, pointy);
        break;

    case 1: // intersection with the right edge
        pointx = qx.rightBound();
        temp = pointx * coef;
        intersect = myIntersection(temp, qy, pointy);
        break;

    case 2: // intersection with the lower edge
        pointy = qy.leftBound();
        temp = pointy / coef;
        intersect = myIntersection(temp, qx, pointx);
        break;

    case 3: // intersection with the upper edge
        pointy = qy.rightBound();
        temp = pointy / coef;
        intersect = myIntersection(temp, qx, pointx);
        break;

    default:
        intersect = false;
        break;
    }

    return intersect;
}


void cuteCalculation(MyInterval x, MyInterval y, MyInterval &qx, MyInterval &qy,
                     int rExp, int bExp)
/* in the paper: bExp = a, rExp = b
 * we are calculating:
 * qx = x^bExp/r^{rExp}, qy = y^bExp/r^{rExp}
 * /////////////////////
 * we have to check corners and intersection of lines
 * * x = +/- sqrt(b/(r-b)) * y
 * * y  = +/- sqrt(b/(r-b)) * x
 * * x = 0
 * * y = 0
 * to find the extremal values//
 */
{
    if (rExp <= bExp)
    {
        cout << "Wrong exponents!";
        exit(1);
    }

    MyInterval r = sqrt(power(x, 2) + power(y, 2));
    MyInterval r3 = power(r, -rExp);
    if (!testIfDoCute(r)) // we do a naive evaluation
    {
        qx = power(x, bExp) * r3;
        qy = power(y, bExp) * r3;
        return;
    }

    MyInterval coef = sqrt(MyInterval(bExp) / (rExp - bExp));
    // coef -- the coefficient of the line on which we are checking the values

    // on output:
    //     qx = MyInterval(qMinX.leftBound(), qMaxX.rightBound());
    //     qy = MyInterval(qMinY.leftBound(), qMaxY.rightBound());
    // MyInterval qMinX, qMaxX, qMinY, qMaxY, qTemp;

    // first we evaluate on the corners of the `x\times y` qube:
    // left-bottom corner
    MyInterval xAcc = x.leftBound();
    MyInterval yAcc = y.leftBound();
    MyInterval tempx;
    MyInterval tempy;
    tempValues2D(xAcc, yAcc, tempx, tempy, rExp, bExp);
    qx = tempx;
    qy = tempy;

    // top-left corner:
    yAcc = y.rightBound();
    tempValues2D(xAcc, yAcc, tempx, tempy, rExp, bExp);
    setMinMax2D(tempx, tempy, qx, qy);

    // top-right corner:
    xAcc = x.rightBound();
    tempValues2D(xAcc, yAcc, tempx, tempy, rExp, bExp);
    setMinMax2D(tempx, tempy, qx, qy);

    // bottom-right corner:
    yAcc = y.leftBound();
    tempValues2D(xAcc, yAcc, tempx, tempy, rExp, bExp);
    setMinMax2D(tempx, tempy, qx, qy);

    // now we have: `qMinX`, `qMaxX`, `qMinY`, `qMaxY` -- min and max from the corners
    MyInterval xI, yI;
    // intersection of the line `y = x*coef` and the edges of the qube `q`:
    for (int edge = 0; edge < 4; edge++)
    {
        if (intersectEdgeWithLine(coef, x, y, edge, xI, yI))
        {
            tempValues2D(xI, yI, tempx, tempy, rExp, bExp);
            setMinMax2D(tempx, tempy, qx, qy);
        }
    }
    // intersection of `y = -x*coef`
    for (int edge = 0; edge < 4; edge++)
    {
        if (intersectEdgeWithLine(-coef, x, y, edge, xI, yI))
        {
            tempValues2D(xI, yI, tempx, tempy, rExp, bExp);
            setMinMax2D(tempx, tempy, qx, qy);
        }
    }

    MyInterval invCoef = 1 / coef;
    // intersection of `y = x/coef`
    for (int edge = 0; edge < 4; edge++)
    {
        if (intersectEdgeWithLine(invCoef, x, y, edge, xI, yI))
        {
            tempValues2D(xI, yI, tempx, tempy, rExp, bExp);
            setMinMax2D(tempx, tempy, qx, qy);
        }
    }

    // intersection of `y = -x/coef`
    for (int edge = 0; edge < 4; edge++)
    {
        if (intersectEdgeWithLine(-invCoef, x, y, edge, xI, yI))
        {
            tempValues2D(xI, yI, tempx, tempy, rExp, bExp);
            setMinMax2D(tempx, tempy, qx, qy);
        }
    }

    // intersection of `x = 0`
    if (isSingular(x))
    {
        xAcc = 0.0;
        yAcc = y.leftBound();
        tempValues2D(xAcc, yAcc, tempx, tempy, rExp, bExp);
        setMinMax2D(tempx, tempy, qx, qy);

        yAcc = y.rightBound();
        tempValues2D(xAcc, yAcc, tempx, tempy, rExp, bExp);
        setMinMax2D(tempx, tempy, qx, qy);
    }
    // intersection of `y = 0`
    if (isSingular(y))
    {
        yAcc = 0.0;
        xAcc = x.leftBound();
        tempValues2D(xAcc, yAcc, tempx, tempy, rExp, bExp);
        setMinMax2D(tempx, tempy, qx, qy);

        xAcc = x.rightBound();
        tempValues2D(xAcc, yAcc, tempx, tempy, rExp, bExp);
        setMinMax2D(tempx, tempy, qx, qy);
    }

    //qx = MyInterval(qMinX.leftBound(), qMaxX.rightBound());
    //qy = MyInterval(qMinY.leftBound(), qMaxY.rightBound());
}


